@@ -1,9 +1,9 @@
 context("resource-schedule")
 
 test_that("a schedule name conflicts with a generator name", {
-  expect_error(simmer() %>%
+  expect_error(simmer(verbose = TRUE) %>%
     add_generator("asdf", create_trajectory() %>% timeout(0), at(0)) %>%
-    add_resource("asdf", schedule(c(1 ,2), c(1, 1)))
+    add_resource("asdf", schedule(c(1, 2), c(1, 1)))
   )
 })
 
@@ -26,7 +26,7 @@ test_that("capacity & queue size change", {
   expect_equal(limits$time, c(8, 16, 24))
   expect_equal(limits$server, c(1, 2, 3))
 
-  limits <- simmer() %>%
+  limits <- simmer(verbose = TRUE) %>%
     add_resource("dummy", fin_sch) %>%
     run(17) %>% reset() %>% run(49) %>%
     get_mon_resources("limits")
@@ -39,7 +39,7 @@ test_that("queue size changes", {
   inf_sch <- schedule(c(8, 16, 24), c(1, 2, 3), Inf)
   fin_sch <- schedule(c(8, 16, 24), c(1, 2, 3), 24)
 
-  limits <- simmer() %>%
+  limits <- simmer(verbose = TRUE) %>%
     add_resource("dummy", 1, inf_sch) %>%
     run(17) %>% reset() %>% run(49) %>%
     get_mon_resources("limits")
@@ -47,7 +47,7 @@ test_that("queue size changes", {
   expect_equal(limits$time, c(8, 16, 24))
   expect_equal(limits$queue, c(1, 2, 3))
 
-  limits <- simmer() %>%
+  limits <- simmer(verbose = TRUE) %>%
     add_resource("dummy", 1, fin_sch) %>%
     run(17) %>% reset() %>% run(49) %>%
     get_mon_resources("limits")
@@ -65,7 +65,7 @@ test_that("arrivals 1) are dequeued when resource's capacity increases and
 
   inf_sch <- schedule(c(0, 1, 2), c(1, 3, 1), Inf)
 
-  arrivals <- simmer() %>%
+  arrivals <- simmer(verbose = TRUE) %>%
     add_resource("dummy", inf_sch) %>%
     add_generator("asdf", t, at(0, 0, 0)) %>%
     run() %>%
@@ -84,7 +84,7 @@ test_that("arrivals 1) are dequeued when resource's capacity increases and
 
   inf_sch <- schedule(c(0, 1, 2), c(1, 3, 1), Inf)
 
-  arrivals <- simmer() %>%
+  arrivals <- simmer(verbose = TRUE) %>%
     add_resource("dummy", inf_sch) %>%
     add_generator("asdf", t, at(0, 0, 0)) %>%
     run() %>%
@@ -102,7 +102,7 @@ test_that("arrivals are preempted when resource's capacity decreases", {
 
   inf_sch <- schedule(c(0, 1, 2), c(1, 3, 1), Inf)
 
-  arrivals <- simmer() %>%
+  arrivals <- simmer(verbose = TRUE) %>%
     add_resource("dummy", inf_sch, preemptive = TRUE) %>%
     add_generator("asdf", t, at(0, 0, 0), restart = TRUE) %>%
     run() %>%
@@ -118,7 +118,7 @@ test_that("resource's capacity decreases before post-release tasks", {
     timeout(5) %>%
     release("t-rex")
 
-  arrivals <- simmer() %>%
+  arrivals <- simmer(verbose = TRUE) %>%
     add_resource("t-rex", capacity = schedule(timetable = c(5, 10, 15),
                                               period = Inf,
                                               values = c(1, 0, 1))) %>%
