@@ -8,17 +8,17 @@ test_that("a simple deterministic simulation with rejections behaves as expected
     timeout(function() 1.5) %>%
     release("server", 1)
 
-  env <- simmer() %>%
+  env <- simmer(verbose = TRUE) %>%
     add_resource("server", 1, queue_size = 0) %>%
     add_generator("entity", t0, at(1:n))
 
   expect_warning(env %>% add_generator("entity", t0, function() 2))
   expect_equal(env %>% now(), 0)
-  expect_equal(env %>% peek(), 1)
+  expect_equal(env %>% peek(), 0)
 
-  env %>% onestep() %>% onestep()
+  env %>% run(1.5)
 
-  expect_equal(env %>% now(), 1)
+  expect_equal(env %>% now(), 1.5)
   expect_equal(env %>% peek(), 2)
 
   env %>% run()
